import torch
from msprobe.core.common.const import Const
from msprobe.pytorch.free_benchmark import logger
from msprobe.pytorch.free_benchmark.common.constant import CommonField
from msprobe.pytorch.free_benchmark.common.enums import PerturbationMode
from msprobe.pytorch.free_benchmark.common.params import DataParams
from msprobe.pytorch.free_benchmark.perturbed_layers.npu.npu_base_layser import (
    NpuBaseLayer,
)


class ImprovePrecisionLayer(NpuBaseLayer):

    def improve_tensor_precision(self, tensor_obj):
        if (
            isinstance(tensor_obj, torch.Tensor)
            and torch.is_floating_point(tensor_obj)
            and tensor_obj.dtype not in [torch.float32, torch.float64]
        ):
            self._set_improve_valus(tensor_obj)
            tensor_obj = self._change_dtype(tensor_obj)
            self.is_added = True
            return tensor_obj
        if isinstance(tensor_obj, dict):
            return {
                key: self.improve_tensor_precision(value)
                for key, value in tensor_obj.items()
            }
        if isinstance(tensor_obj, (tuple, list)):
            return type(tensor_obj)(
                [self.improve_tensor_precision(value) for value in tensor_obj]
            )
        return tensor_obj

    def handle(self, params: DataParams) -> torch.Any:
        logger.info_on_rank_0(
            f"[msprobe] Free benchmark: Perturbation is "
            f"{PerturbationMode.IMPROVE_PRECISION} of {self.api_name}."
        )
        new_args = self.improve_tensor_precision(params.args)
        if params.fuzz_stage == Const.BACKWARD:
            new_kwargs = {}
        else:
            new_kwargs = self.improve_tensor_precision(params.kwargs)
        # 如果输入中全为高精度、应跳过二次执行、减少多余显存引用
        if not self.is_added:
            return params.perturbed_result
        if "inplace" in new_kwargs:
            new_kwargs["inplace"] = False
        params.perturbed_result = params.origin_func(*new_args, **new_kwargs)
        return params.perturbed_result

    def _set_improve_valus(self, inputs):
        if inputs.dtype in [torch.float16, torch.bfloat16]:
            self.perturbed_value = torch.float32

    def _change_dtype(self, inputs):
        if hasattr(inputs, CommonField.DEVICE):
            device = inputs.device
            if device is CommonField.META:
                new_inputs = inputs.to(
                    device=CommonField.META, dtype=self.perturbed_value
                )
            else:
                new_inputs = inputs.to(dtype=self.perturbed_value).to(device)
        else:
            new_inputs = inputs.to(dtype=self.perturbed_value)
        return new_inputs
